package edu.upvehu.gbg.tap.calculadora;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * <p>Controladora de calculadora sencilla.</p>
 *
 * @author GBG  (german.bordel@ehu.es)
 * @version 20221116
 */
public final class CalcuControl {
    /** Referencia al GUI */
    private CalcuGUI theGui;
    /** El display a mostrara en el GUI */
    private String display;
    /** La expresión a mostrara en el GUI */
    private String expresion;
    /** El acumulador contiene el resultado de operaciones anteriores ya resueltas */
    private double acumulador;
    /** Memoria de la última tecla. */
    private String lastKey;
    /** Memoria del último operador. */
    private String lastOp;

    /**
     * Genera un controlador de calculadora para el GUI proporcionado
     * @param gui El GUI al que se asocia este controlador.
     */
    CalcuControl(CalcuGUI gui) {
        theGui=gui;
        //Hacemos un reset del estado de la calculadora
        reset();
        //mostramos el estado del display y la expresion
        gui.setExpression(expresion);
        gui.setDisplay(display);
        //y suscribimos como listener de todos los botones
        gui.subscribirABotones(
                new ActionListener() {

            /**
             * Implementación del interfaz ActionListener: atención a teclas del
             * GUI
             *
             * @param evt el evento recibido desde el GUI
             */
            @Override
            public void actionPerformed(ActionEvent evt) {
                // Recogemos la tecla del boton fuente de la interrupción.
                String key = ((javax.swing.JButton) evt.getSource()).getText();
                // Y actuamos en consecuencia en cada caso.
                switch (key) {
                    // TECLAS NUMÉRICAS. El punto se admite sólo una vez.
                    case ".":
                        if (display.contains(".")) {
                            break;
                        }
                    case "0":
                    case "1":
                    case "2":
                    case "3":
                    case "4":
                    case "5":
                    case "6":
                    case "7":
                    case "8":
                    case "9":
                        // Tras un "=" o un "0" en display, parten de una expresión vacía
                        if (lastKey.equals("=") || expresion.equals("0")) {
                            expresion = " ";
                        }
                        // Tras un operador o un "0" en el display, parten de un display vacío.
                        if ("=+-x/".indexOf(lastKey.charAt(0)) != -1 || display.equals("0")) {
                            display = "";
                        }
                        //En todo caso se acumula la tecla
                        display += key;
                        expresion += key;
                        break;
                    // TECLA "=" ejecuta el cálculo pendiente y actualiza valores. No se admite repetida    
                    case "=":
                        if (lastKey.equals("=")) {
                            break;
                        }
                        calcula();
                        lastOp = key;
                        display = String.valueOf(acumulador);
                        expresion += key + acumulador;
                        break;
                    // OPERADORES
                    case "+":
                    case "-":
                    case "x":
                    case "/":
                        // No se aceptan dos operadores seguidos
                        if ("+-x/".indexOf(lastKey.charAt(0)) != -1) {
                            break;
                        } // Si suceden a un "=" se inicia una nueva operación con el resultado de la anterior
                        // como primer operando (está en el acumulador y se mantiene en el display)
                        else if (lastKey.equals("=")) {
                            expresion = acumulador + key;
                        } // En los casos normales se ejecuta el cálculo acumulado y se actualizan valores.
                        else {
                            calcula();
                            expresion += key;
                            display = String.valueOf(acumulador);
                        }
                        // Guardaos el operador actual para cuando llegue el siguiente y pueda resolverse
                        lastOp = key;
                        break;
                    // La tecla CLR hace un reset de la calculadora
                    case "CLR":
                        reset();
                        break;
                    // Cualquier default corresponde a teclas sin contemplar y, por tanto, es un error
                    default:
                        display = expresion = "error";
                        reset();
                }
                // Una vez resuelto el caso, se muestran el display y la expresión y se recuerda la tecla.
                theGui.setExpression(expresion);
                theGui.setDisplay(display);
                lastKey = key;
            }

        }
        );
    }

    /**
     * Reset de la calculadora: expresión vacía y display a cero.
     * (y otras inicializaciones internas)
     */
    private void reset(){
        expresion=" ";
        lastKey=display="0";
        acumulador=0.0; //Acumulador a cero preparado para una suma con el primer número que se introduzca
        lastOp="+";     //La ultima operación a "+" para que el primer operando sume con cero. 
    }

    /**
     * Lleva al acumulador la operación pendiente entre al acumulador y el display.
     */
    private void calcula(){
        switch (lastOp){
            case "+": acumulador+=Double.parseDouble(display);break;
            case "-": acumulador-=Double.parseDouble(display);break;
            case "x": acumulador*=Double.parseDouble(display);break;
            case "/": acumulador/=Double.parseDouble(display);break;
        }
    }
}
